
#######################################################################
# Copyright (c) 2021 Josh Luthy BD Life Sciences - Informatics,
# Ashland, OR
#
# R Script Template Designed and Written by: Josh Luthy and Ian Taylor
#
# License
# The software is distributed under the terms of the 
# Artistic License 2.0
# http://www.r-project.org/Licenses/Artistic-2.0
# 
# Disclaimer
# This software and documentation come with no warranties of any kind.
# This software is provided "as is" and any express or implied 
# warranties, including, but not limited to, the implied warranties of
# merchantability and fitness for a particular purpose are disclaimed.
# In no event shall the  copyright holder be liable for any direct, 
# indirect, incidental, special, exemplary, or consequential damages
# (including but not limited to, procurement of substitute goods or 
# services; loss of use, data or profits; or business interruption)
# however caused and on any theory of liability, whether in contract,
# strict liability, or tort arising in any way out of the use of this 
# software.
######################################################################
# plugin version 5.1.11

# This is a Shiny web application for interacting with visualizations from FlowJo and SeqGeq. 
# You can run the application by clicking the 'Run App' button above within RStudio, or by launching 
#   the "interactive" portion of the ViolinBox plugin.

packages<-function(x){
  x<-as.character(match.call()[[2]])
  if(!require(x,character.only=TRUE)){
    install.packages(pkgs=x,repos="http://cran.r-project.org")
    require(x,character.only=TRUE)
  }
}
devPackages<-function(x){
  if (!require(x,character.only=TRUE)){
    suppressMessages(suppressWarnings(devtools::install_github(x)))
  }else{
    require(x,character.only=TRUE)
  }
}

fCorePack<-function(x){
  x<-as.character(match.call()[[2]])
  if (!require(x,character.only=TRUE)){
    BiocManager::install(x)
    require(x,character.only=TRUE)
  }
}

packages(shiny)
packages(shinydashboard)
packages(shinyBS)
packages(shinybusy)
packages(readr)
packages(dplyr)
packages(viridisLite)
packages(DT)
packages(ggplot2)
packages(cowplot)
packages(RColorBrewer)
packages(data.table)
packages(shinyWidgets)
packages(ggsignif)
packages(broom)
packages(ggpubr)
packages(gginnards)
packages(tidyverse)
packages(gtools)
packages(plotly)
packages(heatmaply)
packages(corrr)
packages(seriation)
packages(shinyalert)
#######################
##   Robust Log:     ##
#######################
robustLog2 <- function(x) {
  x <- as.numeric(x)
  if (x >= 0) {
    log(x + 1, 2)
  } else {
    0
  }
}
#######################
## Global Settings:  ##
#######################
# inCSVPath <- "/Users/10201397/Desktop/Shiny_ViolinBox/extData/B1.smallPop.catPar.ExtNode.csv"
inCSVPath <- "FJ_DATA_FILE_PATH"
catParRaw <- "FJ_CAT_PAR"
popMap <- "FJ_CATPOP_MAP"


if(catParRaw != "{}" && catParRaw != ""){
  popMap <- gsub(", ", "C=O=D=E=X", popMap)
  popMap <- gsub(" ", "_", popMap)
  popMap <- gsub("C=O=D=E=X", ", ", popMap)
  popMap <- gsub("\\.", "_", popMap)
  popMap <- gsub("\\-", "neg", popMap)
  popMap <- gsub("\\+", "pos", popMap)
  # popMap <- gsub("\\=", "Eq", popMap)
}

df <- fread(inCSVPath, sep = ",", header = TRUE, check.names = FALSE)
df <- as.data.frame(df)
#remove and duplicate columns from 'Seurat Marker Features' gene set.
df <- df %>% subset(., select = which(!duplicated(names(.))))

if(!nrow(df) > 0){
  print("Unable to read the input file correctly, try again.")
  stopAndReturnError("Unable to read the input file correctly.", call. = FALSE)
}

if ("EventNumberDP" %in% colnames(df)){
  useLog10 <- T
} else {
  useLog10 <- F
}

if (useLog10) {
  robustLog2 <- function(x) {
    x <- as.numeric(x)
    if (x >= 0) {
      log(x + 1, 10)
    } else {
      0
    }
  }
} else {
  robustLog2 <- function(x) {
    x <- as.numeric(x)
    if (x >= 0) {
      log(x + 1, 2)
    } else {
      0
    }
  }
}


## We need to make sure that there are or are not categorical parameters
noKatPar <- TRUE
if(catParRaw != "[]" && catParRaw != ""){
  catParam <- gsub("(\\[)","",gsub("(\\])","",catParRaw))
  catPar <- as.symbol(catParam)
  noKatPar <- FALSE
}

## Detect if this is FJ or SG
if ("EventNumberDP" %in% colnames(df)){
  print("EventNumber Detected - This is FlowJo")
  df <- df %>%
    dplyr::select(EventNumberDP, everything())
  colnames(df) <- gsub( "(^.*)( :: )" , "", colnames(df), ignore.case = TRUE )
  isSeqGeq <- F
  df$EventNumberDP <- NULL
  maxS <- round(robustLog2(max(df)))
  minS <- round(robustLog2(min(df)))
} else {
  print("No EventNumber Detected - We're in SeqGeq")
  isSeqGeq <- T
  maxS <- round(robustLog2(max(df)))
  minS <- round(robustLog2(min(df)))
}



if (!noKatPar){

  ## Remove the "." dots from categorical parameters too...
  catParamTemp <- gsub("\\.","_",catParam)
  catParamTemp <- gsub("\\s","_",catParamTemp)
  catParamTemp <- gsub("\\=", "Eq", catParamTemp)
  catParamTemp <- gsub("\\-", "neg", catParamTemp)
  catParamTemp <- gsub("\\+", "pos", catParamTemp)
  names(df)[names(df) == catParam] <- catParamTemp
  catParam <- catParamTemp

  ## SET UP the POP NAMES
  hasPopMap <- F
  if(popMap != "" && popMap != "{}" && popMap != "[]"){
    hasPopMap <- T

    popMapNew <- gsub("\\}","",gsub("\\{","",gsub("\\s","",popMap)))
    splitMap <- as.data.frame(strsplit(popMapNew,","))
    dataMap <- data.frame( Value = character(),
                           Name = character(),
                           stringsAsFactors = F )

    for(x in 1:nrow(splitMap)){
      xthSet <- as.data.frame(strsplit(as.character(splitMap[x,]),"="))
      dataMap[x,] <- data.frame(Value = as.character(round(as.numeric(as.character(xthSet[1,])),3) ), ## sig figs set to three to match what java is doing. NOTE: This is all for the sake of FlowSOM
                                Name = as.character(xthSet[2,]),
                                stringsAsFactors = F)
    }

    categorical <- df[catParam]
    newCategorical <- data.frame("xyz" = character(),
                                 stringsAsFactors = F)
    colnames(newCategorical) <- catParam

    newCategorical <- dataMap$Name[match(unlist(categorical), dataMap$Value)]
    newCategorical <- as.data.frame(newCategorical)
    ## Below is commented out because it slows the app down
    #for (i in 1:nrow(categorical)) {
    #    flag = 1
    #    for(j in 1:nrow(dataMap)) {
    #
    #        if(flag > 0){
    #            placeholder = "UNCALLED_CAT"      ## This will allow us to check if the categorical was found
    #            catSwap = categorical[i,]         ## If that category isn't found, this will allow us to put something sensible there
    #        }
    #
    #        if(categorical[i,] == dataMap[j,1]){
    #            placeholder = dataMap[j,2]
    #            flag = 0
    #        }
    #
    #        if(flag > 0 && placeholder == "UNCALLED_CAT"){              ## Assuming that some round of the 'j' loop finds no categorical match, we will (for a moment) automatically call the category as unknown...
    #            newCategorical[i,] = paste0(catParam,"_",catSwap)
    #        } else if (flag == 0 && placeholder != "UNCALLED_CAT"){     ## ... However, assuming that we eventually find a matching categorical, we will name that cell's population correctly!
    #            newCategorical[i,] <- placeholder
    #        }
    #    }
    #}

    df[catParam] <- newCategorical
  }
}


#############################
## Set Quantile Scaling Up ##
#############################
dfdf <- df
if (!noKatPar){
  dfdf[catParam] <- NULL
}

quantile_breaks <- function(xs, n = 10) {
  breaks <- stats::quantile(xs, probs = seq(0, 1, length.out = n))
  breaks[!duplicated(breaks)]
}
dfMat <- as.matrix(dfdf)
mat_breaks <- quantile_breaks(dfMat, n = 11)
rm(dfdf) # free up mem
rm(dfMat) # free up mem

# Parameter Selector Names
paramNames <- colnames(df)
if (!noKatPar){
  paramNames <- paramNames[paramNames != catParRaw] # we do not want the categorical param in the param selector
  paramNames <- paramNames[paramNames != catParam]
}
paramNames <- paramNames[paramNames != "SampleID"]

hmPalette <- c("magma", "plasma", "inferno", "viridis", "cividis") ##, "rainbow")
vbPalette <- c("Dark2","Accent","Pastel2","Set2", "Pastel1", "Set1","Paired","Set3","Spectral","BrBG","PiYG","PRGn","PuOr")
correction <- c("bonferroni", "fdr")
plotTypes <- c("violin","jitter","box")

#######################
## Half Violin Plot: ##
#######################

"%||%" <- function(a, b) {
  if (!is.null(a))
    a
  else
    b
}

geom_flat_violin <-
  function(mapping = NULL,
           data = NULL,
           stat = "ydensity",
           position = "dodge",
           trim = TRUE,
           scale = "area",
           show.legend = NA,
           inherit.aes = TRUE,
           ...) {
    ggplot2::layer(
      data = data,
      mapping = mapping,
      stat = stat,
      geom = GeomFlatViolin,
      position = position,
      show.legend = show.legend,
      inherit.aes = inherit.aes,
      params = list(trim = trim,
                    scale = scale,
                    ...)
    )
  }

#' @rdname ggplot2-ggproto
#' @format NULL
#' @usage NULL
#' @export
GeomFlatViolin <-
  ggproto(
    "GeomFlatViolin",
    Geom,
    setup_data = function(data, params) {
      data$width <- data$width %||%
        params$width %||% (resolution(data$x, FALSE) * 0.9)

      # ymin, ymax, xmin, and xmax define the bounding rectangle for each group
      data %>%
        dplyr::group_by(.data = ., group) %>%
        dplyr::mutate(
          .data = .,
          ymin = min(y),
          ymax = max(y),
          xmin = x,
          xmax = x + width / 2
        )
    },

    draw_group = function(data, panel_scales, coord)
    {
      # Find the points for the line to go all the way around
      data <- base::transform(data,
                              xminv = x,
                              xmaxv = x + violinwidth * (xmax - x))

      # Make sure it's sorted properly to draw the outline
      newdata <-
        base::rbind(
          dplyr::arrange(.data = base::transform(data, x = xminv), y),
          dplyr::arrange(.data = base::transform(data, x = xmaxv), -y)
        )

      # Close the polygon: set first and last point the same
      # Needed for coord_polar and such
      newdata <- rbind(newdata, newdata[1,])

      ggplot2:::ggname("geom_flat_violin",
                       GeomPolygon$draw_panel(newdata, panel_scales, coord))
    },

    draw_key = draw_key_polygon,

    default_aes = ggplot2::aes(
      weight = 1,
      colour = "grey20",
      fill = fColor,
      size = 0.5,
      alpha = NA,
      linetype = "solid"
    ),

    required_aes = c("x", "y")
  )


#######################
##   Shiny Part:     ##
#######################

# name the clusters:
if (!noKatPar){
  preUClust <- unique(as.data.frame(df[catParam]))
  preUClust <- mixedsort(preUClust[,1])
  uClust <- preUClust

} else {
  uClust <- ""
}

parList <- as.list(paramNames)
if (noKatPar){
  selectACat <- "None"
  catParam <- ""
} else {
  selectACat <- catParam
  # for(n in 1:length(parList)){
  #   if(parList[n] == catParam){
  #     parList[n] <- NULL
  #   }
  # }
}

interactiveViolin <- function(var) {
  require(shiny)
  library(dplyr)
  shinyApp(

    #######################
    ##   UI Part:        ##
    #######################
    ui = dashboardPage(
      dashboardHeader(title = "ViolinBox"),
      mysidebar <- dashboardSidebar(
        width = 190,
        sidebarMenu(
          id = 'tabs',
          div(style="padding-left: 25px;",
              actionButton("refreshPlot",
                           label = "Refresh Plot")
          ),
          menuItem(
            text = "Data",
            tabName = "parentData",
            icon = icon("vials"),
            menuSubItem(
              text = "",
              tabName = "data",
              icon = NULL
            ),
            selectInput(
              "Parameters",
              label = "Select Parameters to Plot",
              choices = as.character(paramNames),
              multiple = TRUE,
              selectize = TRUE,
              selected = as.character(paramNames)
              # selected = as.character(parList)
            ),
            bsTooltip("Parameters", "Select Parameters to plot. Recommended to select 6 or less parameters and/or clusters to plot at once for ViolinBox Plots.",
                      "right", options = list(container = "body")),
            selectInput(
              "catParam",
              label = "Select a Categorical Parameter",
              choices = c("None", catParam),
              selected = selectACat
            ),
            bsTooltip("catParam", "Select a Categorical Parameter to make comparisons with ViolinBox Plots and plot in your heatmaps.",
                      "right", options = list(container = "body")),
            selectInput(
              "clusters",
              label = "Choose Clusters to Plot",
              choices = uClust,
              selected = uClust,
              multiple = T,
              selectize = T
            ),
            bsTooltip("Parameters", "Select Parameters to plot. Recommended to select 6 or less parameters and/or clusters to plot at once for ViolinBox Plots.",
                      "right", options = list(container = "body")),
            div(style="padding-bottom: 10px; padding-left: 25px;",
                actionButton("dataTable",label = "View Table"))
            # tags$head(tags$style("dataTable{background-color:#E8E8E8;} dataTable{color: Black;}"))
          ),

          menuItem(
            text = "Heatmaps",
            tabName = "parentHeatmapButton",
            # startExpanded = F,
            icon = icon("chess-board"),
            menuItem(
              text = "",
              tabName = "heatmapButton",
              icon = NULL
              # selected = T
            ),
            selectInput("hmDendro",
                        label = "Choose Dendrograms",
                        choices = c("both","row","column","none"),
                        multiple = FALSE,
                        selected = "column"),
            selectInput("sortMethod",
                        label = "Dendrogram Sort Method",
                        choices = c("OLO", "GW", "mean","none"),
                        multiple = FALSE,
                        selected = "OLO"),
            bsTooltip("sortMethod", "Options include: Optimal Leaf Ordering, Gruvaeus and Wainer heuristic, Marginal Mean.",
                      "right", options = list(container = "body")),
            # numericInput("colorDenRow",
            #              label = "Color Row Dendrograms",
            #              min = 2,
            #              max = 100,
            #              value = 2),
            # numericInput("colorDenCol",
            #              label = "Color Row Dendrograms",
            #              min = 2,
            #              max = 100,
            #              value = 2),
            selectInput("scaleHM",
                        label = "Scale by:",
                        choices = c("row", "column", "none"),
                        selected = "none"),
            bsTooltip("scaleHM", "Select to center and scale along rows or columns.",
                      "right", options = list(container = "body")),
            checkboxInput("roundHeatmaps",
                          label = "Bubble Heatmap",
                          FALSE),
            bsTooltip("roundHeatmaps", "Display heatmap as dots with sizes based on frequency of expression over zero. (Not appropriate for protein measurements)",
                      "right", options = list(container = "body")),
            checkboxInput("orderGenes",
                          label = "Order Parameters",
                          FALSE),
            bsTooltip("orderGenes", "Order parameters using methods listed below.",
                      "right", options = list(container = "body")),
            selectInput("seriateGenes",
                        label = "Ordering Method",
                        choices = c("BEA","BEA_TSP","PCA","PCA_angle","Max"),
                        multiple = FALSE,
                        selectize = TRUE,
                        selected = "Max"),
            # bsTooltip("seriateGenes", "TODO",
            #           "right", options = list(container = "body")),
            checkboxInput("logScale",
                          label = "Log Normalization",
                          FALSE),
            bsTooltip("logScale", "Apply Log Normalization to the data matrix",
                      "right", options = list(container = "body")),
            checkboxInput("quantile",
                          label = "Quantile Scaling",
                          FALSE),
            bsTooltip("quantile", "Change color scale by attempting to place same number of events per color break",
                      "right", options = list(container = "body")),
            checkboxInput("flipHM",
                          label = "Transpose",
                          FALSE),
            selectInput("hmPalette",
                        label = "Select Heatmap Colors",
                        choices = hmPalette,
                        multiple = FALSE,
                        selectize = TRUE),
            div(style="padding-bottom: 10px; padding-left: 25px",
                downloadButton("download_hm_plot", "Download HTML", class = "hmButton")),
            tags$head(tags$style("hmButton{background-color:#E8E8E8;} hmButton{color: Black;}")),
            div(style="padding-bottom: 3px; padding-left: 10px",
                actionButton("download_hm_plot_png", "Download PNG", style = "fill",size = "sm")),
            tags$head(tags$style("hmButton22{background-color:#E8E8E8;} hmButton22{color: Black;}")),
            div(style="padding-bottom: 10x; padding-left: 10px",
                actionButton("show", "Help")),
            tags$head(tags$style("helpButton{background-color:#E8E8E8;} helpButton{color: Black;}"))
          ),

          menuItem(
            text = "Rainclouds",
            tabName = "parentViolins",
            startExpanded = F,
            icon = icon("soundcloud"),
            menuSubItem(
              text = "",
              tabName = "violins",
              icon = NULL
              # selected = T
            ),
            selectInput(
              "plotType",
              label = "Choose Plot Types",
              choices = plotTypes,
              multiple = TRUE,
              selectize = TRUE,
              selected = "violin"
            ),
            bsTooltip("plotType", "Choose to add additional layers to the plot as boxes or jitter points.",
                      "right", options = list(container = "body")),
            selectInput("vbPalette",
                        label = "Select Color Palette",
                        choices = vbPalette,
                        multiple = FALSE,
                        selectize = TRUE,
                        selected = "Accent"
            ),
            sliderInput("alpha",
                        label = "Opacity",
                        min = 0.1,
                        max = 1.1,
                        step = 0.1,
                        value = 1.0
            ),
            bsTooltip("alpha", "Adjust the opacity of violins and jitter points.",
                      "right", options = list(container = "body")),
            checkboxInput("flipViolins",
                          label = "Flip Violins",
                          FALSE),
            checkboxInput("darkTheme",
                          label = "Dark Theme",
                          FALSE),
            checkboxInput("overlay",
                          label = "Overlay Plots",
                          F),
            checkboxInput("grpByParam",
                          label = "Group By Parameter",
                          TRUE),
            bsTooltip("grpByParam", "Uncheck to group by categorical parameter.",
                      "right", options = list(container = "body")),
            checkboxInput("statCompare",
                          label = "Statistical Comparison",
                          F),
            checkboxInput("addLegend",
                          label = "Show Legend",
                          F),
            # selectInput("correction",
            #             choices = correction,
            #             label = "Stat Correction",
            #             multiple = F,
            #             selected = "fdr")
            div(style="padding-bottom: 10px; padding-left: 25px",
                downloadButton("download_VBP_plots", "Download Plot", class = "vbpButton")),
            tags$head(tags$style("vbpButton{background-color:#E8E8E8;} vbpButton{color: Black;}"))
          ),
          menuItem(
            text = "Correlation_Plots",
            tabName = "corrPlots",
            icon = icon("th"),
            menuSubItem(
              text = "",
              tabName = "correlate",
              icon = NULL
            ),
            # selectInput("dstMeth",
            #             label = "Choose distance method",
            #             choices = distMethods,
            #             selected = "euclidean"),
            # bsTooltip("dstMeth", "Choose which method to use for calculating distances between data points.",
            #           "right", options = list(container = "body")),
            # selectInput("corrDends",
            #             label = "Show dendrograms?",
            #             choices = c("both", "none"),
            #             selected = "none"),
            # bsTooltip("corrDends", "Choose which dendrograms to draw. Select 'none' to order by selected parameters.",
            #           "right", options = list(container = "body")),
            checkboxInput("revCorrCols",
                          label = "Reverse columns",
                          value = FALSE),
            # checkboxInput("corrAsDots",
            #               label = "Correlation as Dots",
            #               value = FALSE),
            # bsTooltip("corrAsDots", "Display correlation plot as dots with sizes based on p-values",
            #           "right", options = list(container = "body")),
            selectInput("corrPalette",
                        label = "Select a color palette for heatmap",
                        choices = hmPalette,
                        width = '400px',
                        multiple = FALSE,
                        selectize = TRUE),
            div(style="padding-bottom: 10px; padding-left: 25px",
                downloadButton("download_corr_plots", "Download Plot", class = "corButton")),
            tags$head(tags$style("corButton{background-color:#E8E8E8;} corButton{color: Black;}"))
          ),
          menuItem(
            text = "Options",
            tabName = "options",
            icon = icon("cog"),
            menuSubItem(
              text = "",
              tabName = "opts",
              icon = NULL
              # selected = T
            ),
            numericInput(
              "smallfonts",
              label = "Parameter Font Size:",
              min = 1,
              max = 42,
              value = 12
            ),
            bsTooltip("smallfonts", "Change the size of Parameter name text",
                      "right", options = list(container = "body")),
            numericInput(
              "bigfonts",
              label = "Axis Title Font Size:",
              min = 1,
              max = 42,
              value = 20
            ),
            bsTooltip("bigfonts", "Change the size of Axis Title text",
                      "right", options = list(container = "body")),
            sliderInput(
              "flipAngle",
              label = "Axis Text Angle:",
              min = 0,
              max = 360,
              step = 15,
              value = 90
            ),
            numericInput(
              "tilefonts",
              label = "Title Font Size:",
              min = 1,
              max = 42,
              value = 20
            ),
            numericInput(
              "legendfonts",
              label = "Legend Font Size:",
              min = 1,
              max = 42,
              value = 11
            ),
            numericInput(
              "statfonts",
              label = "Stats Font Size:",
              min = 1,
              max = 12,
              value = 4
            ),
            numericInput(
              "maxScale",
              label = "Maximum",
              min = -6,
              max = 25,
              value = maxS
            ),
            numericInput(
              "minScale",
              label = "Minimum",
              min = -7,
              max = 24,
              value = minS
            )
          )
        )
      ),

      dashboardBody(
        ## These CSS sytles will giv us the full viewport height for graphs specified:
        tags$head(tags$style("#heatmapButton{height:90vh !important;}")),
        tags$head(tags$style("#violins{height:90vh !important;}")),
        tags$head(tags$style("#correlation1{height:90vh !important;}")),

        tabItems(
          tabItem(tabName = "data",
                  DTOutput(outputId = "table1"),
                  useShinyalert(),
                  downloadButton("downloadData", "Download", class = "myButton"),
                  tags$head(tags$style(".myButton{background-color:#E8E8E8;} .myButton{color: Black;}"))),


          tabItem(tabName = "heatmapButton",
                  add_busy_spinner(spin = "atom",
                                   color = "#112446",timeout = 100,
                                   position = "top-left", margins = c(100,270),
                                   height = "80px", width = "80px"),
                  plotlyOutput("heatmapButton"),
                  tags$script('
                  document.getElementById("download_hm_plot_png").onclick = function() {
                  var plot = $("#selectplot").val();
                  var gd = document.getElementById("heatmapButton");

                  Plotly.Snapshot.toImage(gd, {format:"png",height:1000,width:1400 }).once("success", function(url) {
                    var a = window.document.createElement("a");
                    a.href = url;
                    a.type = "image/png";
                    a.download = "Heatmap.png";
                    document.body.appendChild(a);
                    a.click();
                    document.body.removeChild(a);
                  });
                  }
                  ')),
          # tags$head(tags$style(".download_hm_plot_png{background-color:#E8E8E8;} .download_hm_plot_png{color: Black;}"))),

          tabItem(tabName = "violins",
                  add_busy_spinner(spin = "flower",
                                   color = "#112446",timeout = 100,
                                   position = "top-left", margins = c(100,270),
                                   height = "80px", width = "80px"),
                  plotOutput("violins"),
                  actionBttn("setNames", "Rename Axes",style = "fill",size="sm",color="default")),
          # addSpinner(plotOutput("violins"), spin = "folding-cube", color = "#4DAF4A"),
          tabItem(tabName = "correlate",
                  add_busy_spinner(spin = "self-building-square",
                                   color = "#112446",timeout = 100,
                                   position = "top-left", margins = c(100,270),
                                   height = "80px", width = "80px"),
                  plotlyOutput("correlation1"))
        ),

        tags$head(
          tags$link(rel = "icon", href = "http://fjinstallers.s3-us-west-2.amazonaws.com/Plugins/violin.png"),
          tags$style(
            HTML(
              '

      /* logo */
      .skin-blue .main-header .logo {
                            background-color: #696969;
      }

      #                       /* logo when hovered */
      # .skin-blue .main-header .logo:hover {
      #                       background-color: ##696969;
      #                       }

      /* navbar (rest of the header) */
      .skin-blue .main-header .navbar {
                            background-color: #696969;
                            }

      /* main sidebar */
      .skin-blue .main-sidebar {
                            background-color: #696969;
                            }

      /* active selected tab in the sidebarmenu */
      .skin-blue .main-sidebar .sidebar .sidebar-menu .active a{
                            background-color: #b6a4b6;
                            }

      /* other links in the sidebarmenu */
      .skin-blue .main-sidebar .sidebar .sidebar-menu a{
                            background-color: #b0b0b0;
                            color: #000000;
                            }

      /* other links in the sidebarmenu when hovered */
       .skin-blue .main-sidebar .sidebar .sidebar-menu a:hover{
                            background-color: #ff69b4;
                            }
      /* toggle button when hovered  */
       .skin-blue .main-header .navbar .sidebar-toggle:hover{
                            background-color: #ff69b4;
                            }
      /* body */
        .content-wrapper, .right-side {
                            background-color: #DCDCDC;
                            }
    '
            )
          )
        )

      )
    ),

    #######################
    ##   Server Part:    ##
    #######################
    server = function(input, output, session) {
      # This will check the status of the menuItem and Auto-Select the subMenuItem if its menuItem is selected
      observeEvent(input$sidebarItemExpanded, {
        test <- input$sidebarItemExpanded
        if (test == "Rainclouds") {
          updateTabItems(session, "tabs", "violins")
        } else if (test == "Heatmaps") {
          updateTabItems(session, "tabs", "heatmapButton")
        } else if (test == "Correlation_Plots" ){
          updateTabItems(session,"tabs","correlate")
        }
      })

      observeEvent(input$dataTable,{
        updateTabItems(session,"tabs","data")
      })

      # This will return the df for heatmaps
      filterData <- reactive({
        if (input$catParam == "None") {
          df2 <- df %>%
            select(input$Parameters)
          df2 <- round(df2, 2)
          return(df2)
        } else{
          # This takes input from Categorical Param Selector for group_by func
          groupedInput <- df %>%
            dplyr::filter(eval(parse(text = input$catParam)) %in% input$clusters) %>%
            group_by_at(vars(input$catParam)) %>%
            summarise_all(mean)

          # This makes sure the names of populations are added to the data frame (if we have them). And that they're ordered the way one would hope (natural order)
          # namerss <- namerss[order(nchar(namerss), namerss)]                                 ## order names that will be used as the row names momentarily

          groupedInput <-
            groupedInput[mixedorder(groupedInput[[input$catParam]]), ]           ## order rows based on the categorical parameter
          namers <- groupedInput[, input$catParam]
          namerss <-
            (apply(namers, 2, function(x)
              gsub(' ', '_', x)))
          rm(namers) # free up mem
          groupedInput[input$catParam] <- NULL
          groupedInput <- round(groupedInput, 2)
          groupedInput <-
            select(groupedInput, input$Parameters)
          rownames(groupedInput) <- namerss

          # GI <- select(groupedInput, input$Parameters) # This is much easier to implement without having to remove unwanted columns from above code ex
          # GI <- round(GI,2)
          # rownames(GI) <- GI[input$catParam]
          # rownames(GI) <- namerss
          # GI[input$catParam] <- NULL

          # flipHM

          # return(GI)
          return(groupedInput)
        }

      })


      # This will allow for download of filterData() as .csv file
      output$downloadData <- downloadHandler(
        filename = function() {
          paste("ViolinBox Plugin Table Export-", Sys.Date(), ".csv", sep="")
        },
        content = function(file) {
          write.csv(filterData(), file)
        },
        contentType = "text/csv"
      )


      # This will return the df used for Violin Plots
      filterData2 <- reactive({
        if (input$catParam == "None") {
          dfnoCat <- select(df, input$Parameters)
          # Use gather function to quickly create the data frame.
          violinPlotDF <- tidyr::gather(dfnoCat,key = "Gene",value = "Log2Expression",1:ncol(dfnoCat))
          tempDF <- sapply(violinPlotDF$Log2Expression, robustLog2)
          tempDF <- data.frame(matrix(unlist(tempDF),nrow = length(tempDF),byrow = T))
          violinPlotDF <- cbind(violinPlotDF, tempDF)
          violinPlotDF <- violinPlotDF[, -2]
          colnames(violinPlotDF)[2] <- "Log2Expression"

          violinPlotDF$Log2Expression <- as.numeric(violinPlotDF$Log2Expression)
          violinPlotDF$Gene <- as.factor(violinPlotDF$Gene)
          VBPnoCat <- violinPlotDF[violinPlotDF$Gene == input$Parameters,]
          return(VBPnoCat)
        } else {
          # Lets take the catParam column and move it to the front of the DF before running gather()
          df01 <- dplyr::select(df, c(input$Parameters, input$catParam))
          dfCat2 <- df01 %>%
            dplyr::select(input$catParam, everything()) %>%
            dplyr::filter(eval(parse(text = input$catParam)) %in% input$clusters)
          dfCat2 <- dfCat2[mixedorder(dfCat2[[input$catParam]]), ]

          # Use gather function to quickly create the data frame.
          violinPlotDF <<- tidyr::gather(dfCat2, key = "Gene", value = "Log2Expression", 2:ncol(dfCat2))
          if (ncol(violinPlotDF) < 3) {
            print("Try again")
            stopAndReturnError("Unable to read the inputs correctly, check parameter names.",
                               call. = FALSE)
          }
          # can use sapply to apply the robustlog2 to Values
          tempDF <- sapply(violinPlotDF$Log2Expression, robustLog2)
          tempDF <- data.frame(matrix(unlist(tempDF), nrow = length(tempDF), byrow = T))
          violinPlotDF <- cbind(violinPlotDF, tempDF)
          violinPlotDF <- violinPlotDF[, -3]
          colnames(violinPlotDF)[3] <- "Log2Expression"

          # set up the DF for plotting - Including ordering of factors with levels
          violinPlotDF$Log2Expression <- as.numeric(violinPlotDF$Log2Expression)
          uFact <- unique(as.matrix(violinPlotDF[input$catParam]))
          f <- as.factor(gsub("\\s*", "", as.matrix(violinPlotDF[input$catParam])))
          violinPlotDF[input$catParam] <- factor(f, levels = uFact, ordered = T)
          violinPlotDF$Gene <- as.factor(violinPlotDF$Gene)

          # group by param
          if (input$grpByParam == TRUE && input$overlay == F) {
            # if(input$overlay == F) {
            colnames(violinPlotDF) <-c("Gene",as.character(input$catParam),"Log2Expression")

            VBP <<- violinPlotDF[violinPlotDF[[input$catParam]] == input$Parameters,]
            VBP$Gene <- gsub("_"," ",VBP$Gene)
            VBP[[input$catParam]] <- gsub("_", " ", VBP[[input$catParam]])
          } else{
            VBP <<- violinPlotDF[violinPlotDF$Gene == input$Parameters,]
            VBP[[input$catParam]] <- gsub("_", " ", VBP[[input$catParam]])
            # VBP$Gene <<- gsub("_"," ",VBP$Gene)
          }
          maxS <- round(max(violinPlotDF$Log2Expression))
          minS <- round(min(violinPlotDF$Log2Expression))

          return(VBP)
        }
      })

      observeEvent(input$hmDendro,{
        activateOrderBool <- T
        if(input$hmDendro != "none"){
          activateOrderBool <- F
        }
        updateCheckboxInput(session, "orderGenes", value = activateOrderBool)

      })

      observeEvent(input$orderGenes, {
        if(input$orderGenes){
          updateSelectInput(session, "hmDendro", selected = "none")
        }
      })

      observeEvent(input$show, {
        showNotification("To Save as HTML First Download Pandoc Application Here: ",
                         a(href="https://pandoc.org/installing.html", "pandoc.org")) ## target="_blank">pancod.org</a>)
      })

      # This will plot heatmaps
      heatmap1 <- eventReactive(input$refreshPlot, {

        trueFilteredData = filterData()
        rownames(trueFilteredData) <- gsub("_"," ", rownames(trueFilteredData))

        if (input$quantile) {
          paletteScale = eval(parse(text = input$hmPalette))(length(mat_breaks) - 1)
        } else {
          paletteScale = eval(parse(text = input$hmPalette))(n = 512,
                                                             begin = 0.15,
                                                             end = 0.9)
        }

        if (input$logScale && isSeqGeq) {
          trueFilteredData <- apply(filterData(), c(1,2), robustLog2)
        } else if (input$logScale && !isSeqGeq){
          trueFilteredData <- apply(filterData(), c(1,2), robustLog2)#robustLog2(filterData() + 0.0001)
        }

        if (input$flipHM && input$catParam == "None") {
          trueFilteredData <- trueFilteredData
        } else if(input$flipHM && !input$catParam == "None") {
          trueFilteredData <- t(trueFilteredData)
        }

        namersss <<- rownames(trueFilteredData)
        ord = "euclidean"
        #################
        ## if SeqGeq   ##
        #################
        if(isSeqGeq) {
          if (input$catParam == "None") {
            # perform clustering so a smaller heatmap can be created.
            fit <- kmeans(trueFilteredData, 30) # TODO allow user input of cluster number. Default is 30.
            # get cluster means
            aggregate(trueFilteredData,by=list(fit$cluster),FUN=mean)
            # append cluster assignment
            trueFilteredData <- data.frame(trueFilteredData, fit$cluster)
            # change colname to 'clusters'
            colnames(trueFilteredData)[which(names(trueFilteredData) == "fit.cluster")] <- "clusters"

            clusterMatrix <- trueFilteredData %>%
              group_by_at(vars(clusters)) %>%
              summarise_all(mean)

            clusterMatrix["clusters"] <- NULL
            clusterMatrix <- round(clusterMatrix, 2)
            # pheatmap::pheatmap(
            #   trueFilteredData,
            #   color = paletteScale,
            #   border_color = "grey20",
            #   main = "",
            #   show_rownames = TRUE,
            #   show_colnames = TRUE,
            #   fontsize_col = input$smallfonts,
            #   angle_col = input$textAngle,
            #   fontsize_row = input$smallfonts,
            #   kmeans_k = 30,
            #   scale = input$scaleHM,
            #   # labels_row = namersss,
            #   clustering_distance_rows = ord,
            #   clustering_distance_cols = ord,
            #   # cluster_rows = clusterD,
            #   # cluster_cols = clusterP
            # )
            if(input$roundHeatmaps && !input$orderGenes){
              hm <- heatmaply(
                clusterMatrix,
                color = paletteScale,
                grid_gap = 0,
                main = "K-means Clustering Heatmap",
                # show_rownames = TRUE,
                # show_colnames = TRUE,
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                node_type = "scatter",
                grid_size = 3,
                point_size_mat = clusterMatrix,
                scale = input$scaleHM,
                dendrogram = input$hmDendro,
                seriate = input$sortMethod,
                dist_method = ord,
                margins = c(50,50,30,0)
                # clustering_distance_cols = ord,
                # cluster_rows = clusterD,
                # cluster_cols = clusterP
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000))
            }
            if(!input$roundHeatmaps && !input$orderGenes) {
              hm <- heatmaply(
                clusterMatrix,
                color = paletteScale,
                grid_gap = 0.5,
                main = "K-means Clustering Heatmap",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                scale = input$scaleHM,
                dendrogram = input$hmDendro,
                seriate = input$sortMethod,
                dist_method = ord,
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000))

            }
            if(input$orderGenes && !input$roundHeatmaps){
              if (input$seriateGenes == "Max"){
                goodCNames <- colnames(clusterMatrix)
                newTest <- data.frame(clusterMatrix)
                colnames(newTest) <- goodCNames
                checknames <- goodCNames
                newNames <- goodCNames
                NewIx <- newTest
                colcounter = 1
                for (j in 1:nrow(newTest)) {
                  yoRow <- rownames(newTest[j, ])
                  for (i in 1:ncol(newTest)) {
                    testColsRow <- rownames(newTest[which.max(newTest[, i]), ])
                    if (yoRow == testColsRow) {
                      NewIx[, colcounter] <- newTest[, i]
                      newNames[colcounter] <- goodCNames[i]
                      checknames <- checknames[ checknames != goodCNames[i]  ]
                      colcounter = colcounter + 1
                    }
                  }
                }
                trueFilteredData2 <- clusterMatrix[,newNames]
                rownames(trueFilteredData2) <- rownames(clusterMatrix)
              } else {
                orderdData <- seriate(as.matrix(clusterMatrix),method = input$seriateGenes)
                orderD <- as.data.frame(orderdData[[2]])
                colnames(orderD) <- "sortOrder"
                myNewOrder <- rownames(orderD)
                trueFilteredData2 <- clusterMatrix[,myNewOrder]
                trueFilteredData2 <- as.data.frame(trueFilteredData2)
              }

              hm <- heatmaply(
                trueFilteredData2,
                color = paletteScale,
                grid_gap = 0.5,
                main = "K-means Clustering Heatmap",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                scale = input$scaleHM,
                dist_method = "euclidean",
                dendrogram = "none",
                seriate = "none",
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000))
            }
            if(input$orderGenes && input$roundHeatmaps){
              if (input$seriateGenes == "Max"){
                goodCNames <- colnames(clusterMatrix)
                newTest <- data.frame(clusterMatrix)
                colnames(newTest) <- goodCNames
                checknames <- goodCNames
                newNames <- goodCNames
                NewIx <- newTest
                colcounter = 1
                for (j in 1:nrow(newTest)) {
                  yoRow <- rownames(newTest[j, ])
                  for (i in 1:ncol(newTest)) {
                    testColsRow <- rownames(newTest[which.max(newTest[, i]), ])
                    if (yoRow == testColsRow) {
                      NewIx[, colcounter] <- newTest[, i]
                      newNames[colcounter] <- goodCNames[i]
                      checknames <- checknames[ checknames != goodCNames[i]  ]
                      colcounter = colcounter + 1
                    }
                  }
                }
                trueFilteredData2 <- clusterMatrix[,newNames]
                rownames(trueFilteredData2) <- rownames(clusterMatrix)
              } else {
                orderdData <- seriate(as.matrix(clusterMatrix),method = input$seriateGenes)
                orderD <- as.data.frame(orderdData[[2]])
                colnames(orderD) <- "sortOrder"
                myNewOrder <- rownames(orderD)
                trueFilteredData2 <- clusterMatrix[,myNewOrder]
                trueFilteredData2 <- as.data.frame(trueFilteredData2)
              }
              hm <- heatmaply(
                trueFilteredData2,
                color = paletteScale,
                grid_gap = 0,
                main = "K-means Clustering Heatmap",
                # show_rownames = TRUE,
                # show_colnames = TRUE,
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                # k_row = 10,
                # k_col = 10,
                # kmeans_k = 30,
                node_type = "scatter",
                grid_size = 3,
                point_size_mat = trueFilteredData2,
                scale = input$scaleHM,
                dendrogram = input$hmDendro,
                seriate = input$sortMethod,
                # labels_row = namersss, # this is named automatically from df
                dist_method = ord,
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000))
            }
            return(hm)
          } else {
            # get the counts of cells per group(clusters or catParam)
            groupedInputTally <- df %>%
              dplyr::filter(eval(parse(text = input$catParam)) %in% input$clusters) %>%
              group_by_at(vars(input$catParam)) %>%
              summarise(cell_count = n())
            groupedInputTally[,1] <- NULL # remove (soon to be) redundant column

            # get the counts of cells expressing greater than a value of zero
            groupedInputSummary <- df %>%
              dplyr::filter(eval(parse(text = input$catParam)) %in% input$clusters) %>%
              group_by_at(vars(input$catParam)) %>%
              summarise_at(vars(input$Parameters), function(x) sum(x > 0))

            combinedTable <- cbind(groupedInputSummary, groupedInputTally)
            percentExp <- combinedTable %>%
              mutate(across(c(input$Parameters),.fns = ~(./cell_count)*100))
            percentExp <- percentExp %>% select(-last_col())
            percentExp[,1] <- NULL
            # The below pheatmap may be used to export a final image of the HMs.
            # pheatmap::pheatmap(
            #   trueFilteredData,
            #   color = paletteScale,
            #   border_color = "grey20",
            #   main = "",
            #   show_rownames = TRUE,
            #   show_colnames = TRUE,
            #   fontsize_col = input$smallfonts,
            #   angle_col = input$textAngle,
            #   fontsize_row = input$smallfonts,
            #   scale = scl,
            #   labels_row = namersss,
            #   clustering_distance_rows = ord,
            #   clustering_distance_cols = ord,
            #   cluster_rows = clusterD,
            #   cluster_cols = clusterP
            # )
            if(input$roundHeatmaps && !input$orderGenes){
              hm <- heatmaply(
                trueFilteredData,
                color = paletteScale,
                main = "",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                scale = input$scaleHM,
                dist_method = ord,
                node_type = "scatter",
                grid_size = 5,
                point_size_mat = percentExp,
                point_size_name = "% Expressing",
                dendrogram = input$hmDendro,
                seriate = input$sortMethod,
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000)) # This will create a 7"x5" image at 300DPI

            }
            if(!input$roundHeatmaps && !input$orderGenes) {
              hm <- heatmaply(
                trueFilteredData,
                color = paletteScale,
                grid_gap = 0.5,
                main = "",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                scale = input$scaleHM,
                dist_method = ord,
                dendrogram = input$hmDendro,
                seriate = input$sortMethod,
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000))
            }
            if(input$orderGenes && !input$roundHeatmaps){
              if (input$seriateGenes == "Max"){
                goodCNames <- colnames(trueFilteredData)
                newTest <- data.frame(trueFilteredData)
                colnames(newTest) <- goodCNames
                checknames <- goodCNames
                newNames <- goodCNames
                NewIx <- newTest
                colcounter = 1
                for (j in 1:nrow(newTest)) {
                  yoRow <- rownames(newTest[j, ])
                  for (i in 1:ncol(newTest)) {
                    testColsRow <- rownames(newTest[which.max(newTest[, i]), ])
                    if (yoRow == testColsRow) {
                      NewIx[, colcounter] <- newTest[, i]
                      newNames[colcounter] <- goodCNames[i]
                      checknames <- checknames[ checknames != goodCNames[i]  ]
                      colcounter = colcounter + 1
                    }
                  }
                }
                trueFilteredData2 <- trueFilteredData[,newNames]
                rownames(trueFilteredData2) <- rownames(trueFilteredData)
              } else {
                orderdData <- seriate(as.matrix(trueFilteredData),method = input$seriateGenes)
                orderD <- as.data.frame(orderdData[[2]])
                colnames(orderD) <- "sortOrder"
                myNewOrder <- rownames(orderD)
                trueFilteredData2 <- trueFilteredData[,myNewOrder]
                trueFilteredData2 <- as.data.frame(trueFilteredData2)
              }

              hm <- heatmaply(
                trueFilteredData2,
                color = paletteScale,
                grid_gap = 0.5,
                main = "",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                scale = input$scaleHM,
                dist_method = "euclidean",
                dendrogram = "none",
                seriate = "none",
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000))
            }
            if(input$orderGenes && input$roundHeatmaps) {
              if (input$seriateGenes == "Max"){
                goodCNames <- colnames(trueFilteredData)
                newTest <- data.frame(trueFilteredData)
                colnames(newTest) <- goodCNames
                checknames <- goodCNames
                newNames <- goodCNames
                NewIx <- newTest
                colcounter = 1
                for (j in 1:nrow(newTest)) {
                  yoRow <- rownames(newTest[j, ])
                  for (i in 1:ncol(newTest)) {
                    testColsRow <- rownames(newTest[which.max(newTest[, i]), ])
                    if (yoRow == testColsRow) {
                      NewIx[, colcounter] <- newTest[, i]
                      newNames[colcounter] <- goodCNames[i]
                      checknames <- checknames[ checknames != goodCNames[i]  ]
                      colcounter = colcounter + 1
                    }
                  }
                }
                trueFilteredData2 <- trueFilteredData[,newNames]
                rownames(trueFilteredData2) <- rownames(trueFilteredData)
              } else {
                orderdData <- seriate(as.matrix(trueFilteredData),method = input$seriateGenes)
                orderD <- as.data.frame(orderdData[[2]])
                colnames(orderD) <- "sortOrder"
                myNewOrder <- rownames(orderD)
                trueFilteredData2 <- trueFilteredData[,myNewOrder]
                trueFilteredData2 <- as.data.frame(trueFilteredData2)
              }
              hm <- heatmaply(
                trueFilteredData2,
                color = paletteScale,
                main = "",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                scale = input$scaleHM,
                dist_method = ord,
                node_type = "scatter",
                grid_size = 5,
                point_size_mat = percentExp,
                point_size_name = "% Expressing",
                dendrogram = input$hmDendro,
                seriate = input$sortMethod,
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000)) # This will create a 7"x5" image at 300DPI
            }
            return(hm)
          }
        } else {
          ##############
          ## FlowJo   ##
          ##############
          if (input$catParam == "None") {
            # perform clustering so a smaller heatmap can be created.
            fit <- kmeans(trueFilteredData, 30) # TODO allow user input of cluster number. Default is 30.
            # get cluster means
            aggregate(trueFilteredData,by=list(fit$cluster),FUN=mean)
            # append cluster assignment
            trueFilteredData <- data.frame(trueFilteredData, fit$cluster)
            # change colname to 'clusters'
            colnames(trueFilteredData)[which(names(trueFilteredData) == "fit.cluster")] <- "clusters"

            clusterMatrix <- trueFilteredData %>%
              group_by_at(vars(clusters)) %>%
              summarise_all(mean)

            clusterMatrix["clusters"] <- NULL
            clusterMatrix <- round(clusterMatrix, 2)

            if(input$roundHeatmaps && !input$orderGenes){
              hm <- heatmaply(
                clusterMatrix,
                color = paletteScale,
                grid_gap = 0,
                main = "K-means Clustering Heatmap",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                node_type = "scatter",
                grid_size = 3,
                point_size_mat = clusterMatrix,
                scale = input$scaleHM,
                dendrogram = input$hmDendro,
                seriate = input$sortMethod,
                dist_method = ord,
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000))
            }
            if(!input$roundHeatmaps && !input$orderGenes) {
              hm <- heatmaply(
                clusterMatrix,
                color = paletteScale,
                grid_gap = 0.5,
                main = "K-means Clustering Heatmap",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                scale = input$scaleHM,
                dendrogram = input$hmDendro,
                seriate = input$sortMethod,
                dist_method = ord,
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000))

            }
            if(input$orderGenes && !input$roundHeatmaps){
              if (input$seriateGenes == "Max"){
                goodCNames <- colnames(clusterMatrix)
                newTest <- data.frame(clusterMatrix)
                colnames(newTest) <- goodCNames
                checknames <- goodCNames
                newNames <- goodCNames
                NewIx <- newTest
                colcounter = 1
                for (j in 1:nrow(newTest)) {
                  yoRow <- rownames(newTest[j, ])
                  for (i in 1:ncol(newTest)) {
                    testColsRow <- rownames(newTest[which.max(newTest[, i]), ])
                    if (yoRow == testColsRow) {
                      NewIx[, colcounter] <- newTest[, i]
                      newNames[colcounter] <- goodCNames[i]
                      checknames <- checknames[ checknames != goodCNames[i]  ]
                      colcounter = colcounter + 1
                    }
                  }
                }
                trueFilteredData2 <- clusterMatrix[,newNames]
                rownames(trueFilteredData2) <- rownames(clusterMatrix)
              } else {
                orderdData <- seriate(as.matrix(clusterMatrix),method = input$seriateGenes)
                orderD <- as.data.frame(orderdData[[2]])
                colnames(orderD) <- "sortOrder"
                myNewOrder <- rownames(orderD)
                trueFilteredData2 <- clusterMatrix[,myNewOrder]
                trueFilteredData2 <- as.data.frame(trueFilteredData2)
              }

              hm <- heatmaply(
                trueFilteredData2,
                color = paletteScale,
                grid_gap = 0.5,
                main = "K-means Clustering Heatmap",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                scale = input$scaleHM,
                dist_method = "euclidean",
                dendrogram = "none",
                seriate = "none",
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000))
            }
            if(input$orderGenes && input$roundHeatmaps){
              if (input$seriateGenes == "Max"){
                goodCNames <- colnames(clusterMatrix)
                newTest <- data.frame(clusterMatrix)
                colnames(newTest) <- goodCNames
                checknames <- goodCNames
                newNames <- goodCNames
                NewIx <- newTest
                colcounter = 1
                for (j in 1:nrow(newTest)) {
                  yoRow <- rownames(newTest[j, ])
                  for (i in 1:ncol(newTest)) {
                    testColsRow <- rownames(newTest[which.max(newTest[, i]), ])
                    if (yoRow == testColsRow) {
                      NewIx[, colcounter] <- newTest[, i]
                      newNames[colcounter] <- goodCNames[i]
                      checknames <- checknames[ checknames != goodCNames[i]  ]
                      colcounter = colcounter + 1
                    }
                  }
                }
                trueFilteredData2 <- clusterMatrix[,newNames]
                rownames(trueFilteredData2) <- rownames(clusterMatrix)
              } else {
                orderdData <- seriate(as.matrix(clusterMatrix),method = input$seriateGenes)
                orderD <- as.data.frame(orderdData[[2]])
                colnames(orderD) <- "sortOrder"
                myNewOrder <- rownames(orderD)
                trueFilteredData2 <- clusterMatrix[,myNewOrder]
                trueFilteredData2 <- as.data.frame(trueFilteredData2)
              }

              hm <- heatmaply(
                trueFilteredData2,
                color = paletteScale,
                grid_gap = 0,
                main = "K-means Clustering Heatmap",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                node_type = "scatter",
                grid_size = 3,
                point_size_mat = trueFilteredData2,
                scale = input$scaleHM,
                dendrogram = input$hmDendro,
                seriate = input$sortMethod,
                # labels_row = namersss, # this is named automatically from df
                dist_method = ord,
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000))
            }
            return(hm)
          } else {
            # For FlowJo we will not vary dot size based on % cells expressing
            # get the counts of cells per group(clusters or catParam)
            # groupedInputTally <- df %>%
            #   dplyr::filter(eval(parse(text = input$catParam)) %in% input$clusters) %>%
            #   group_by_at(vars(input$catParam)) %>%
            #   summarise(cell_count = n())
            # groupedInputTally[,1] <- NULL # remove (soon to be) redundant column
            #
            # # get the counts of cells expressing greater than a value of zero
            # groupedInputSummary <- df %>%
            #   dplyr::filter(eval(parse(text = input$catParam)) %in% input$clusters) %>%
            #   group_by_at(vars(input$catParam)) %>%
            #   summarise_at(vars(input$Parameters), function(x) sum(x > 0))
            #
            # combinedTable <- cbind(groupedInputSummary, groupedInputTally)
            # percentExp <- combinedTable %>%
            #   mutate(across(c(input$Parameters),.fns = ~(./cell_count)*100))
            # percentExp <- percentExp %>% select(-last_col())
            # percentExp[,1] <- NULL

            if(input$roundHeatmaps && !input$orderGenes){
              hm <- heatmaply(
                trueFilteredData,
                color = paletteScale,
                main = "",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                scale = input$scaleHM,
                dist_method = ord,
                node_type = "scatter",
                grid_size = 5,
                point_size_mat = trueFilteredData,
                # point_size_name = "% Expressing",
                dendrogram = input$hmDendro,
                seriate = input$sortMethod,
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000)) # This will create a 7"x5" image at 300DPI

            }
            if(!input$roundHeatmaps && !input$orderGenes) {
              hm <- heatmaply(
                trueFilteredData,
                color = paletteScale,
                grid_gap = 0.5,
                main = "",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                scale = input$scaleHM,
                dist_method = ord,
                dendrogram = input$hmDendro,
                seriate = input$sortMethod,
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000))
            }
            if(input$orderGenes && !input$roundHeatmaps){
              if (input$seriateGenes == "Max"){
                goodCNames <- colnames(trueFilteredData)
                newTest <- data.frame(trueFilteredData)
                colnames(newTest) <- goodCNames
                checknames <- goodCNames
                newNames <- goodCNames
                NewIx <- newTest
                colcounter = 1
                for (j in 1:nrow(newTest)) {
                  yoRow <- rownames(newTest[j, ])
                  for (i in 1:ncol(newTest)) {
                    testColsRow <- rownames(newTest[which.max(newTest[, i]), ])
                    if (yoRow == testColsRow) {
                      NewIx[, colcounter] <- newTest[, i]
                      newNames[colcounter] <- goodCNames[i]
                      checknames <- checknames[ checknames != goodCNames[i]  ]
                      colcounter = colcounter + 1
                    }
                  }
                }
                trueFilteredData2 <- trueFilteredData[,newNames]
                rownames(trueFilteredData2) <- rownames(trueFilteredData)
              } else {
                orderdData <- seriate(as.matrix(trueFilteredData),method = input$seriateGenes)
                orderD <- as.data.frame(orderdData[[2]])
                colnames(orderD) <- "sortOrder"
                myNewOrder <- rownames(orderD)
                trueFilteredData2 <- trueFilteredData[,myNewOrder]
                trueFilteredData2 <- as.data.frame(trueFilteredData2)
              }

              hm <- heatmaply(
                trueFilteredData2,
                color = paletteScale,
                grid_gap = 0.5,
                main = "",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                scale = input$scaleHM,
                dist_method = "euclidean",
                dendrogram = "none",
                seriate = "none",
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000))
            }
            if(input$orderGenes && input$roundHeatmaps) {
              if (input$seriateGenes == "Max"){
                goodCNames <- colnames(trueFilteredData)
                newTest <- data.frame(trueFilteredData)
                colnames(newTest) <- goodCNames
                checknames <- goodCNames
                newNames <- goodCNames
                NewIx <- newTest
                colcounter = 1
                for (j in 1:nrow(newTest)) {
                  yoRow <- rownames(newTest[j, ])
                  for (i in 1:ncol(newTest)) {
                    testColsRow <- rownames(newTest[which.max(newTest[, i]), ])
                    if (yoRow == testColsRow) {
                      NewIx[, colcounter] <- newTest[, i]
                      newNames[colcounter] <- goodCNames[i]
                      checknames <- checknames[ checknames != goodCNames[i]  ]
                      colcounter = colcounter + 1
                    }
                  }
                }
                trueFilteredData2 <- trueFilteredData[,newNames]
                rownames(trueFilteredData2) <- rownames(trueFilteredData)
              } else {
                orderdData <- seriate(as.matrix(trueFilteredData),method = input$seriateGenes)
                orderD <- as.data.frame(orderdData[[2]])
                colnames(orderD) <- "sortOrder"
                myNewOrder <- rownames(orderD)
                trueFilteredData2 <- trueFilteredData[,myNewOrder]
                trueFilteredData2 <- as.data.frame(trueFilteredData2)
              }
              hm <- heatmaply(
                trueFilteredData2,
                color = paletteScale,
                main = "",
                fontsize_col = input$smallfonts,
                column_text_angle = input$flipAngle,
                fontsize_row = input$smallfonts,
                scale = input$scaleHM,
                dist_method = ord,
                node_type = "scatter",
                grid_size = 5,
                point_size_mat = trueFilteredData2,
                # point_size_name = "% Expressing",
                dendrogram = input$hmDendro,
                seriate = input$sortMethod,
                margins = c(50,50,30,0)
              ) %>% config(toImageButtonOptions = list(
                format = "svg",
                width = 1400,
                height = 1000)) # This will create a 7"x5" image at 300DPI
            }
            # return(hm)
          }
          return(hm)
        }
        # return(hm)
      })

      output$heatmapButton <- renderPlotly({
        req(input$refreshPlot)
        heatmap1()
      })

      output$download_hm_plot <- downloadHandler(
        filename = function() {
          paste("Heatmap Plot-", Sys.Date(), ".html", sep = "")
        },
        content = function(file) {
          # export plotly html widget as a temp file to download.
          saveWidget(as_widget(heatmap1()), file, selfcontained = TRUE)
        }
      )
      # output$download_hm_plot_png <- downloadHandler(
      #   filename = function() {
      #     paste("Heatmap pngPlot-", Sys.Date(), ".png", sep = "")
      #   },
      #   content = function(file) {
      #     # export plotly html widget as a temp file to download.
      #     # saveWidget(as_widget(heatmap1()), file, selfcontained = TRUE)
      #     plotly_IMAGE(heatmap1(), file,format = "png", out_file = "myPNG.png")
      #   },
      #   contentType = "image/png"
      # )
      observeEvent(input$setNames, {
        shinyalert(html = TRUE, text = tagList(
          textInput("xAxisLabel", "Set X Axis Name", "Cluster"),
          textInput("yAxisLabel", "Set Y Axis Name", "Log Expression"),
        ))
      })


      # Raincloud Plots!
      violinPlots <- eventReactive(input$refreshPlot, {

        ########################
        ##   Plot Options:    ##
        ########################
        if (input$flipViolins == TRUE) {
          parFlipGraph <- TRUE
        } else{
          parFlipGraph <- FALSE
        }
        if (input$darkTheme == TRUE) {
          parDarkTheme <- TRUE
        } else{
          parDarkTheme <- FALSE
        }

        if ("jitter" %in% input$plotType) {
          ## %>% contains( "jitter" )){
          parJitter <- TRUE
        } else{
          parJitter <- FALSE
        }
        if ("violin" %in% input$plotType) {
          ## %>% contains ( "violin")){
          parViolin <- TRUE
        } else{
          parViolin <- FALSE
        }
        if ("box" %in% input$plotType) {
          ## %>% contains ( "box")){
          parBox <- TRUE
        } else{
          parBox <- FALSE
        }
        if (input$overlay == TRUE) {
          parOverlayPlots <- TRUE
        } else{
          parOverlayPlots <- FALSE
        }

        ##########################
        ##  Set Color Palette:  ##
        ##########################

        if (input$catParam == "None" && !input$statCompare) {
          numUniqueParams <- length(unique(filterData2()[, 1]))
        } else if (input$statCompare) {
          numUniqueParams <-
            length(unique(filterData2()[, 1])) + length(unique(filterData2()[, 2]))
        } else {
          if (input$grpByParam == TRUE) {
            ## || input$overlay == T){
            numUniqueParams <-
              length(unique(filterData2()[, 2]))
          } else{
            numUniqueParams <- length(unique(filterData2()[, 1]))
          }
        }

        ## These color palettes generator requires a minimum number of parameters:
        if (numUniqueParams < 6) {
          numUniqueParams <- 6
        }
        getPalette <- colorRampPalette(brewer.pal(8, input$vbPalette))(numUniqueParams)
        scaleShade <- scale_colour_manual(values = getPalette)
        scaleFill <-  scale_fill_manual(values = getPalette)
        ylabel <-	input$yAxisLabel #ylab('Log Expression')
        xlabel <-  input$xAxisLabel#xlab('xLabel')

        #######################
        ## Make it Rain:     ##
        #######################
        makeItRain <- function(p) {
          if (parViolin && parJitter && parBox) {
            print("All the graphix!")
            p <- p + violinPart + jitterPart + boxPart +
              scaleShade + scaleFill + xlabel + ylabel + guide
          } else if (parViolin && parBox && !parJitter) {
            print("Weather calls for boxy clouds")
            p <- p + violinPart + boxPart +
              scaleShade + scaleFill + guide + xlabel + ylabel
          } else if (parViolin && parJitter && !parBox) {
            print("Jittery Violins - aka Staccato")
            p <- p + violinPart + jitterPart +
              scaleShade + scaleFill + guide + xlabel + ylabel
          } else if (parViolin && !parBox && !parJitter) {
            print("Violin Solo")
            p <- p + violinSoloPart +
              scaleShade + scaleFill + guide + xlabel + ylabel
          } else if (parJitter && parBox && !parViolin) {
            print("Box and Whisker with a bit of Jitter")
            p <- p + jitterPart + boxPart +
              scaleShade + scaleFill + guide + xlabel + ylabel
          } else if (parJitter && !parViolin && !parBox) {
            print("Jitter Only?! Go Home.")
            p <- p + jitterPart +
              scaleShade + scaleFill + guide + xlabel + ylabel
          } else if (parBox && !parJitter && !parViolin) {
            print("Box and Whisker Only")
            p <- p + boxPart +
              scaleShade + scaleFill + guide + xlabel + ylabel
          } else{
            print("NO GRAPH OPTIONS SELECTED")
            p <- p +
              scaleShade + scaleFill + guide + xlabel + ylabel
          }
          return (p)
        }


        ###############################################
        # Setting Default Variables for 'Nicer' Plots #
        ###############################################

        inputAlpha <- input$alpha

        rainGap <- 0.12
        rainDropSize <- 2
        jitterWidth <- 0.05
        jitterHeight <- 1.2
        boxWidth <- 0.2
        boxNudgeX <- -0.12
        boxNudgeY <- 0
        boxDodgeW <- 0.2
        violinWidth <- 1.25
        # TODO make these below into boolean flags/buttons/checkbox for user

        if (parDarkTheme) {
          jColor <- "WHITE"
          fColor <- "BLACK"
          lColor <- "white"
        } else {
          jColor <- "BLACK"
          fColor <- "WHITE"
          lColor <- "black"
        }

        if (parFlipGraph) {
          # flipAngle <- 0
          theme01 <-
            theme(
              text = element_text(size = input$bigfonts),
              axis.text.x = element_text(
                angle = 0,
                vjust = 0.5,
                hjust = 0.95,
                size = input$smallfonts,
                colour = jColor
              ),
              axis.text.y = element_text(size = input$smallfonts, colour = jColor),
              strip.text = element_text(size = input$tilefonts),
              legend.text = element_text(size = input$legendfonts, colour = jColor),
              legend.key = element_rect(fill = fColor, colour = NULL),
              axis.title.x = element_text(colour = jColor, size = input$bigfonts),
              axis.title.y = element_text(colour = jColor, size = input$bigfonts)
            )
        } else{
          # flipAngle <- 90
          theme01 <-
            theme(
              text = element_text(size = input$bigfonts),
              axis.text.x = element_text(
                angle = input$flipAngle,
                vjust = 0.5,
                hjust = 0.95,
                size = input$smallfonts,
                colour = jColor
              ),
              axis.text.y = element_text(size = input$smallfonts, colour = jColor),
              strip.text = element_text(size = input$tilefonts),
              legend.text = element_text(size = input$legendfonts, colour = jColor),
              legend.key = element_rect(fill = fColor, colour = NULL),
              axis.title.x = element_text(colour = jColor, size = input$bigfonts),
              axis.title.y = element_text(colour = jColor, size = input$bigfonts)
            )
        }

        p <- ggplot(filterData2(),aes(x = Gene,
                                      y = Log2Expression,
                                      colour = Gene
        ),
                    text)

        ##########################
        ##     Plot Types:      ##
        ##########################
        if (input$catParam != "None") {
          fillerParam <- filterData2()[[input$catParam]]
          AES <- aes(fill = fillerParam)
        } else {
          fillerParam <- filterData2()[[1]]
          AES <- aes(fill = fillerParam, colour = fillerParam)

        }

        ## Note: We create the overlay in these plots!
        violinSoloPart <- geom_violin(
          AES,
          position = position_nudge(x = .2, y = 0),
          adjust = 2,
          trim = FALSE,

          alpha = inputAlpha,

          colour = NA,
          width = 1.3
        )

        violinPart <- geom_flat_violin(
          AES,
          position = position_nudge(x = rainGap, y = 0),
          adjust = 2,
          trim = FALSE,

          alpha = inputAlpha,

          colour = NA,
          width = violinWidth
        )

        if (parJitter && parBox && !parViolin) {
          jitterPart <- geom_jitter(
            aes(
              x = Gene,
              y = Log2Expression,
              fill = fillerParam
            ),
            position = position_jitter(width = jitterWidth),
            size = rainDropSize,

            alpha = inputAlpha,

            colour = jColor,
            size = 0.1
          )

        } else if (parJitter && !parBox && !parViolin) {
          jitterPart <- geom_point(
            aes(
              x = Gene,
              y = Log2Expression,
              fill = fillerParam
            ),

            alpha = inputAlpha,
            position = position_jitter(width = jitterWidth, height = jitterHeight),
            size = rainDropSize,
            shape = 21,
            alpha = inputAlpha,

            colour = jColor,
            size = 0.1
          )
        } else{
          jitterPart <-
            geom_point(
              aes(
                # x = as.numeric(Gene) - .15,
                x = Gene,
                y = Log2Expression,
                fill = fillerParam
              ),
              alpha = inputAlpha,
              position = position_jitter(width = jitterWidth, height = jitterHeight),
              size = rainDropSize,
              shape = 21,
              alpha = inputAlpha,
              colour = jColor,
              size = 0.1
            )
        }
        if (parJitter && parBox && !parViolin) {
          boxPart <- geom_boxplot(
            aes(
              x = Gene,
              y = Log2Expression,
              fill = fillerParam
            ),
            outlier.colour = NA,
            width = boxWidth,
            position = position_dodge(preserve = "total", width = boxDodgeW),
            colour = jColor,
            size = 0.1
          )
        } else{
          boxPart <-
            geom_boxplot(
              aes(
                x = Gene,
                y = Log2Expression,
                fill = fillerParam
              ),
              position = position_dodge(preserve = "total", width = boxDodgeW),
              outlier.shape = NA,
              width = boxWidth,
              colour = jColor,
              size = 0.1
            )
        }

        # # misc:
        # if (input$grpByParam && input$catParam == "None") {
        #   xLabel = "Parameters"
        # } else if (input$grpByParam && input$catParam != "None"){
        #   xLabel <- "Clusters"
        # } else if (!input$grpByParam && input$catParam != "None"){
        #   xLabel <- "Parameters"
        # } else if (!input$grpByParam && input$catParam == "None"){
        #   xLabel <- "Parameters"
        # }

        ylabel <-	ylab(input$yAxisLabel)
        xlabel <-  xlab(input$xAxisLabel)

        if (!noKatPar) {
          ttle <- as.character(input$catParam)
        } else {
          ttle <- "Log Expression"
        }
        if (input$grpByParam == T) {
          ttle <- "Parameter"
          if (input$overlay) {
            if (!noKatPar) {
              ttle <- as.character(input$catParam)
            } else {
              ttle <- "Log Expression"
            }
          }
        }

        ###  TODO - Add Stat Comparisons:
        ##     * Remove non-statistically significant comparisons
        #################################
        ## Add Statistical Comparisons ##
        #################################
        if (input$statCompare) {
          if (input$catParam != "None") {
            maximus <- max(filterData2()[[3]])
            minimus <- min(filterData2()[[3]])
          } else{
            maximus <- max(filterData2()[[2]])
            minimus <- min(filterData2()[[2]])
          }

          factor <- "Gene"
          groups <- as.character(unique(filterData2()[[factor]]))
          expand.grid.unique <-
            function(x, y, include.equals = FALSE) {
              x <- unique(x)
              y <- unique(y)
              g <- function(i) {
                z <- setdiff(y, x[seq_len(i - include.equals)])
                if (length(z))
                  cbind(x[i], z, deparse.level = 0)
              }
              do.call(rbind, lapply(seq_along(x), g))
            }

          combs <- as.data.frame(expand.grid.unique(groups, groups),
                                 stringsAsFactors = FALSE)
          my.comps <- as.data.frame(t(combs), stringsAsFactors = FALSE)
          colnames(my.comps) <- NULL
          rownames(my.comps) <- NULL
          my_comparisons <- as.list(my.comps)
          height.y = c((maximus + 0.05 * maximus),
                       (maximus + 0.11 * maximus),
                       (maximus + 0.17 * maximus))
          symn <-
            list(
              cutpoints = c(0, 0.0001, 0.001, 0.01),
              symbols = c("****", "***", "**", "*")
            )
          # symn <- list(cutpoints = c(0, 0.0001, 0.001, 0.01, 0.05), symbols = c("****", "***", "**", "*"))

          # if (!parOverlayPlots) {
          if (!parFlipGraph) {
            p <- p + stat_compare_means(
              comparisons = my_comparisons,
              size = input$statfonts,
              label = "p.signif",
              p.adjust.method = input$correction,
              hide.ns = TRUE,
              symnum.args <- symn,
              na.rm = TRUE,
              step.increase = 0.2,
              show.legend = F,
              inherit.aes = T,
              color = jColor
            )
          }

          if (input$catParam != "None") {
            ## Adjust the position of the KW value
            xkw <- 1.4
            if (!parOverlayPlots) {
              ykw <- minimus - 0.35 * maximus
            } else {
              # ykw <- minimus+0.01*maximus
              ykw <- minimus - 0.50 * maximus
              xkw <- 0.8
            }

            guide <- guides(fill = FALSE, colour = FALSE)
            ## only run KW test if there's a categorical parameter:
            p <-
              p + ggpubr::stat_compare_means(
                label.x = xkw,
                label.y = (ykw),
                size = input$statfonts,
                colour = jColor,
                p.adjust.method = input$correction,
                show.legend = F,
                inherit.aes = T
              )
            ## With a categorical parameter the y-axis max spacing needs some help:
            # p <- p + scale_y_continuous(expand = expand_scale(mult = c(.1, .2)))
          }
          guide <- guides(fill = FALSE, colour = FALSE)
        }

        if (input$addLegend) {
          guide <-
            guides(fill = guide_legend(title = ttle))     ## , colour = element_text(colour = lColor))
        } else {
          guide <- guides(fill = FALSE, colour = FALSE)
        }
        theme <- theme_cowplot()

        #################################
        ##   Put the Plots Together:   ##
        #################################
        if (!parOverlayPlots && (input$catParam != "None")) {
          p <- p + facet_wrap( ~ .data[[as.character(input$catParam)]])
        }

        p <- p + theme + theme01

        if (parFlipGraph) {
          p <- p + coord_flip()
        }
        if (parDarkTheme) {
          p <-
            p + theme_dark() + theme(
              axis.text.x = element_text(
                angle = input$flipAngle,
                vjust = 0.5,
                hjust = 0.95,
                size = input$smallfonts,
                colour = jColor
              ),
              axis.text.y = element_text(size = input$smallfonts, colour = jColor),
              strip.text = element_text(size = input$tilefonts),
              plot.background = element_rect(fill = fColor),
              legend.title = element_text(colour = jColor, size = input$legendfonts),
              legend.text = element_text(colour = jColor, size = input$legendfonts),
              legend.background = element_rect(fill = fColor),
              panel.background = element_rect(fill = fColor),
              legend.key = element_rect(fill = fColor, colour = NULL),
              axis.title.x = element_text(colour = jColor, size = input$bigfonts),
              axis.title.y = element_text(colour = jColor, size = input$bigfonts)
            )
        }

        if (input$statCompare) {
          p <- makeItRain(p)
        } else {
          p <-
            makeItRain(p) + scale_y_continuous(limits = c(input$minScale, input$maxScale))
        }

        return(p)
      })

      launch.browser = TRUE
      session$onSessionEnded(stopApp)

      output$table1 <- renderDataTable(
        server = TRUE,
        datatable(
          filterData(),
          extensions = 'Buttons',
          plugins = 'natural',
          options = list(
            pageLength = 15,
            info = FALSE,
            columnDefs = list(list(type = 'natural',
                                   targets = '_all')),
            dom = 'Bfrtip',
            buttons = list(
              list(
                extend = "copy",
                text = "COPY",
                title = "ViolinBox FlowJoExchange Plugin - Table Export"
              )
            )
          )
        )
      )

      output$violins <- renderPlot({
        req(input$refreshPlot)
        violinPlots()
      })

      output$download_VBP_plots <- downloadHandler(
        filename = function() {
          paste("ViolinBox Plot-", Sys.Date(), ".pdf", sep = "")
        },
        content = function(file) {
          # export the violinBox plot as a pdf
          ggsave(file, plot = violinPlots(), device = "pdf",
                 height = 6, width = 10, units = "in", dpi = 700)
        }
      )

      #########################################
      # Create Correlation Heatmap Data Frame #
      #########################################
      correlationData <- reactive({
        coreDF <- df %>%
          select(input$Parameters)
        coreDF <- round(coreDF, 2)
        return(coreDF)
      })

      ##########################################
      # Create Function to make correlationsHM #
      ##########################################
      cor.test.p <- function(x){
        FUN <- function(x, y) cor.test(x, y, method = "spearman")[["p.value"]]
        z <- outer(
          colnames(x),
          colnames(x),
          Vectorize(function(i,j) FUN(x[,i], x[,j]))
        )
        dimnames(z) <- list(colnames(x), colnames(x))
        z
      }

      correlationHmFun <- function(dataFrame, distMethd, colorPal, shwDend, revCol){
        # corrMat <- as.matrix(dataFrame)
        library(corrr)
        corDF <- correlate(dataFrame)
        corrMat <- corrr::rearrange(corDF)
        # corrMat <- data.frame(corrMat)
        # colnames(corrMat)

        # if(input$corrAsDots){
        #   p <- cor.test.p(dataFrame)
        #
        #   heatmaply_cor(
        #     cor(corrMat),
        #     colors =  colorPal, #eval(parse(text = input$corrPalette)),
        #     xlab = "Features",
        #     ylab = "Features",
        #     grid_gap = 10,
        #     label_names = c("Y Feature", "X Feature", "Value"),
        #     distfun = "spearman",
        #     # dist_method = distMethd,
        #     dendrogram = shwDend,
        #     revC = revCol,
        #     node_type = "scatter",
        #     point_size_mat = -log10(p),
        #     point_size_name = "-log10-p-value"
        #
        #   )
        # } else {
        heatmaply_cor(
          corrMat,
          # cor(corrMat),
          colors =  colorPal,
          grid_gap = 1,
          label_names = c("Y Feature", "X Feature", "Value"),
          distfun = "spearman",
          # dist_method = distMethd,
          dendrogram = "none",
          hide_colorbar = TRUE,
          # row_side_colors = ,
          revC = revCol,
          fontsize_row = input$smallfonts,
          fontsize_col = input$smallfonts,
          column_text_angle = input$flipAngle,
          labRow = input$Parameters,
          labCol = input$Parameters,
          margins = c(50,50,30,0)
        )
        # }
      }

      correlatPlotting <- eventReactive(input$refreshPlot,{
        req(input$refreshPlot)
        correlationHmFun(dataFrame = correlationData(), distMethd = input$dstMeth,
                         colorPal = eval(parse(text = input$corrPalette)),
                         # shwDend = input$corrDends,
                         revCol = input$revCorrCols) %>% config(toImageButtonOptions = list(
          format = "svg",
          width = 1400,
          height = 1000)) # This will create a 7"x5" image at 300DPI
      })

      output$correlation1 <- renderPlotly({
        req(input$refreshPlot)
        # correlationPlots1()
        correlatPlotting()
      })

      output$download_corr_plots <- downloadHandler(
        filename = function() {
          paste("Correlation Plot-", Sys.Date(), ".html", sep = "")
        },
        content = function(file) {
          # export plotly html widget as a temp file to download.
          saveWidget(as_widget(correlatPlotting()), file, selfcontained = TRUE)
        }
      )

    }
  )
  # options(shiny.launch.browser = .rs.invokeShinyPaneViewer)
}

interactiveViolin(df)
